const BOMB_MAX_TIME: u64 = 40_000;
const BOMB_BLINK_DURATION: u64 = 300;
const BOMB_BLINK_SPEED: u64 = 700;

pub struct Bomb {
    last_up_planted: bool,
    time_bomb_planted: u64,
}

impl Bomb {
    pub fn new() -> Bomb{
        Bomb {
            last_up_planted: false,
            time_bomb_planted: 0,
        }
    }

    pub fn tick(&mut self, upstream: bool, delta_t: u64) -> u8 {
        if upstream != self.last_up_planted {
            // A bomb has been planted or defused reset the timer
            self.time_bomb_planted = 0;
            self.last_up_planted = upstream;
        }

        if upstream {
            self.time_bomb_planted += delta_t;

            let should_blink = (self.time_bomb_planted)
                % (BOMB_BLINK_SPEED * (BOMB_MAX_TIME / self.time_bomb_planted).max(1))
                < BOMB_BLINK_DURATION;

            return if should_blink { 255 } else { 0 };
        }

        return 0;
    }
}