const FLASHED_X2: f64 = 0.00003113;
const FLASHED_X1: f64 = -0.17396053;
const FLASHED_X0: f64 = 244.51964267;

fn interpolate_flashed(time: u64) -> u8 {
    if time == 0 {
        return 0;
    }

    let y = FLASHED_X2 * (time as f64).powi(2) + FLASHED_X1 * (time as f64) + FLASHED_X0;

    return y.min(255.0).round() as u8;
}

fn resolve_time_from_flashed(flashed: u8) -> u64 {
    let discriminant = FLASHED_X1.powi(2) - 4.0 * FLASHED_X2 * (FLASHED_X0 - flashed as f64);

    let x = (-FLASHED_X1 - discriminant.sqrt()) / (2.0 * FLASHED_X2);
    return x.min(5000.0) as u64;
}

pub struct Flash {
    last_up_flashed: u8, // Used to detect changes in upstream flashed
    time_since_last_flashed: u64,
}

impl Flash {
    pub fn new() -> Flash {
        Flash {
            last_up_flashed: 0,
            time_since_last_flashed: 0,
        }
    }

    pub fn tick(&mut self, upstream: u8, delta_t: u64) -> u8 {
        // Count the time since the flash started to fade
        if upstream > 0 && upstream < 255 {
            self.time_since_last_flashed += delta_t;
        }

        if upstream == self.last_up_flashed {
            // Upstream value hasn't changed

            if upstream == 255 {
                // Flash is at max
                return 255;
            } else if upstream == 0 {
                // Flash is off
                return 0;
            } else {
                // Flash is fading
                return interpolate_flashed(self.time_since_last_flashed);
            }
        }

        // Upstream value has changed
        self.time_since_last_flashed = resolve_time_from_flashed(upstream);
        self.last_up_flashed = upstream;
        return upstream;
    }
}
