use super::{flash::Flash, bomb::Bomb,state::State};
use std::{sync::atomic::Ordering, thread};

fn build_led_package(len: u32, color: (u8, u8, u8)) -> Vec<u8> {
    let mut packet: Vec<u8> = vec![1, 1];

    // loop through leds
    for i in 0..len {
        packet.push(i as u8);
        packet.push(color.0);
        packet.push(color.1);
        packet.push(color.2);
    }

    return packet;
}

pub fn led_loop(fps: u32, addr: &str, len: u32, state: State) -> ! {
    let socket = std::net::UdpSocket::bind("0.0.0.0:0").unwrap();
    let sleep_time: u64 = (1000 / fps).into();

    let mut flash = Flash::new();
    let mut bomb = Bomb::new();

    loop {
        thread::sleep(std::time::Duration::from_millis(sleep_time));

        let upstream_flashed = u8::try_from(state.flashed.load(Ordering::Relaxed)).unwrap_or(0);
        let interpolated_flashed = flash.tick(upstream_flashed, sleep_time);

        let planted = state.bomb_planted.load(Ordering::Relaxed);
        // let planted = true;
        let bomb_blink = bomb.tick(planted, sleep_time);

        // Merge flashed and bomb blink
        let color = (interpolated_flashed.max(bomb_blink), interpolated_flashed, interpolated_flashed);

        let packet = build_led_package(len, color);

        socket.send_to(&packet, addr).unwrap();
    }
}
